#include <SoftwareSerial.h>
#include <DFRobotDFPlayerMini.h>
#include <LiquidCrystal.h>
#include <Servo.h>

byte one[8] = {0b00100, 0b00100, 0b00100, 0b00100, 0b00100, 0b00100, 0b00100, 0b00100};
byte two[8] = {0b00000, 0b00100, 0b00100, 0b00100, 0b00100, 0b00100, 0b00100, 0b00100};
byte three[8] = {0b00000, 0b00000, 0b00100, 0b00100, 0b00100, 0b00100, 0b00100, 0b00100};
byte four[8] = {0b00000, 0b00000, 0b00000, 0b00100, 0b00100, 0b00100, 0b00100, 0b00100};
byte five[8] = {0b00000, 0b00000, 0b00000, 0b00000, 0b00100, 0b00100, 0b00100, 0b00100};
byte six[8] = {0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00100, 0b00100, 0b00100};
byte seven[8] = {0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00100, 0b00100};
byte zero[8] = {0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000};

LiquidCrystal lcd(8, 9, 7, 6, 5, 4);
Servo servo1;
Servo servo2;

SoftwareSerial mySerial(10, 11); // RX, TX for DFPlayer
DFRobotDFPlayerMini myDFPlayer;

String voice; // Variable to store incoming command

struct Command {
  const char *trigger;
  int fileIndex;
};

Command commands[] = {
  {"*you suck", 1},
  {"*life is hopeless", 3},
  {"*hi", 2},
  {"*my heart is broken", 4},
  {"*i feel so bad", 5},
  {"*i f***** up", 6},
  {"*you are wrong", 7},
  {"*i am stupid", 8},
  {"*you are terrible", 9},
  {"*shut up", 10},
  {"*i am confused", 11},
  {"*shoot", 12},
  {"*i am getting mad", 13},
  // Add more commands here
};

void setup() {
  lcd.begin(16, 2);
  lcd.createChar(0, zero);
  lcd.createChar(1, one);
  lcd.createChar(2, two);
  lcd.createChar(3, three);
  lcd.createChar(4, four);
  lcd.createChar(5, five);
  lcd.createChar(6, six);
  lcd.createChar(7, seven);
  
  servo1.attach(2);
  servo2.attach(3);
  delay(1000);  // Add a delay to ensure everything is powered and stable
  servo1.write(90);  // Initial position at 90 degrees
  servo2.write(90);  // Initial position at 90 degrees

  pinMode(12, OUTPUT); //LED
  pinMode(13, OUTPUT);

  Serial.begin(9600); // Start serial communication for HC-05
  mySerial.begin(9600); // Start communication for DFPlayer

  if (!myDFPlayer.begin(mySerial)) { // Use softwareSerial for DFPlayer
    Serial.println("Unable to begin:");
    Serial.println("1. Please recheck the connection!");
    Serial.println("2. Please insert the SD card!");
    while(true);
  }

  if (!servo1.attached() || !servo2.attached()) {
      Serial.println("Servo not attached");
      return false;
  }
  Serial.println("Servo online");

  myDFPlayer.volume(30); // Set volume
  Serial.println("DFPlayer Mini online.");
  Serial.println("LCD online");

  randomSeed(analogRead(0));
  displayNeutralFace();
}

void loop() {
  // Check if there is an available byte to read from Bluetooth
  while (Serial.available()) {
    delay(10); // Delay added to make things stable
    char c = Serial.read(); // Conduct a serial read
    if (c == '#') {
      break; // Exit the loop when the '#' is detected
    }
    voice += c; // Append the character to the voice string
  }

  if (voice.length() > 0) {
    voice.trim(); // Trim any whitespace
    voice.toLowerCase(); // Convert the command to lower case
    Serial.println(voice); // Print the voice command (optional)

    for (Command command : commands) {
      if (voice == command.trigger) {
        play(command.fileIndex);
        break;
      }
    }

    voice = ""; // Reset the variable after processing
  }
}



void play(int index) {
  //turn on LED
  analogWrite(12, 255);
  analogWrite(13, 255);
  //move the ears
  for(int i = 90, j = 90; i >= 0 && j <= 180; i--, j++){
    servo1.write(i);
    servo2.write(j);
    delay(50);
  }
  // play the voice
  Serial.print("Playing random file: ");
  Serial.println(index);
  myDFPlayer.playMp3Folder(index);
  delay(200);
  
  //get the duration of the voice, jitter the ear and display voice visulization on LCD
  unsigned long duration = getAudioDuration(index);
  Serial.print("playing time(ms): ");
  Serial.println(duration);
  unsigned long startMillis = millis();
  unsigned long currentMillis = startMillis;
  delay(500);
  while (currentMillis - startMillis < duration) {
    currentMillis = millis();
    int jitter1 = random(-20, 20); // Random jitter between -20 and 20 degrees
    int jitter2 = random(-20, 20);
    servo1.write(0 + jitter1); 
    servo2.write(180 + jitter2);
    displayPattern();
    delay(200); // Small delay for both jitter and display update
  }
  displayNeutralFace();
  // ears move to default position 
  for(int i = 0, j = 180; i <= 90 && j >= 90; i++, j--){
    servo1.write(i);
    servo2.write(j);
    delay(50);
  }
  // turn off LED
  analogWrite(12, 0);
  analogWrite(13, 0);
  
}

void displayNeutralFace(){
  lcd.clear();
  for(int i =0; i < 16; i++){
    lcd.setCursor(i, 0);
    lcd.print("_");
  }
}

void displayPattern() {
  lcd.clear();
  for (int x = 0; x < 16; x++) {
    int bottomRowValue = random(1, 7); 
    int topRowValue = (bottomRowValue != 1) ? 0 : random(0, 7);

    lcd.setCursor(x, 0);
    lcd.write(byte(topRowValue));
    lcd.setCursor(x, 1);
    lcd.write(byte(bottomRowValue));
  }
}

unsigned long getAudioDuration(int fileIndex) {
  switch (fileIndex) {
    case 1:
      return 3500; 
    case 2:
      return 5500; 
    case 3:
      return 3500; 
    case 4:
      return 7500; 
    case 5:
      return 2500;
    case 6:
      return 4500;
    case 7:
      return 3500;
    case 8:
      return 6500;
    case 9:
      return 1500;
    case 10:
      return 2500; 
    case 11:
      return 2500;
    case 12:
      return 2500;
    case 13:
      return 2500;
    default:
      return 2000; 
  }
}
